/** @file
 * Copyright (c) 2016-2018, Arm Limited or its affiliates. All rights reserved.
 * SPDX-License-Identifier : Apache-2.0

 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 **/

#include "val/include/sbsa_avs_val.h"
#include "val/include/sbsa_avs_common.h"
#include "val/include/val_interface.h"

#include "val/include/sbsa_avs_gic.h"
#include "val/include/sbsa_avs_gic_support.h"

#define TEST_NUM   (AVS_GIC_TEST_NUM_BASE + 4)
#define TEST_DESC  "GIC Maintenance Interrupt         "

static uint32_t int_id = 25;

static
void
isr()
{
  uint32_t data;
  uint32_t index = val_pe_get_index_mpid(val_pe_get_mpid());

  /* We received our interrupt, so disable Maintenance interrupt from generating further interrupts */
  data = val_gic_reg_read(ICH_HCR_EL2);
  data &= ~0x7;
  val_gic_reg_write(ICH_HCR_EL2, data);

  val_set_status(index, RESULT_PASS(g_sbsa_level, TEST_NUM, 01));
  val_print(AVS_PRINT_INFO, "\n       Received GIC maintenance interrupt ", 0);
  val_gic_end_of_interrupt(int_id);

  return;
}
static
void
payload()
{

  uint32_t data;
  uint32_t timeout = 0x100000;
  uint32_t index = val_pe_get_index_mpid(val_pe_get_mpid());

  data = val_gic_get_info(GIC_INFO_VERSION);
  if (data < 3) {
     val_set_status(index, RESULT_SKIP(g_sbsa_level, TEST_NUM, 01));
     return;
  }

  val_gic_install_isr(int_id, isr);

  // Write to GIC registers which will generate Maintenance interrupt
  data = val_gic_reg_read(ICH_HCR_EL2);
  data |= 0x7;
  val_gic_reg_write(ICH_HCR_EL2, data);

  while ((--timeout > 0) && (IS_RESULT_PENDING(val_get_status(index))));

  if(timeout == 0){
      val_set_status(index, RESULT_FAIL(g_sbsa_level, TEST_NUM, 01));
      return;
  }
  return;
}

uint32_t
g004_entry(uint32_t num_pe)
{

  uint32_t status = AVS_STATUS_FAIL;

  num_pe = 1;  //This GIC test is run on single processor

  status = val_initialize_test(TEST_NUM, TEST_DESC, num_pe, g_sbsa_level);
  if (status != AVS_STATUS_SKIP)
      val_run_test_payload(TEST_NUM, num_pe, payload, 0);

  /* get the result from all PE and check for failure */
  status = val_check_for_error(TEST_NUM, num_pe);

  val_report_status(0, SBSA_AVS_END(g_sbsa_level, TEST_NUM));

  return status;
}
